import httplib
import json
import urllib

from socket.web import WebSocket
from socket.io.exceptions import SocketIOException


def create_connection(address, timeout=None, **extra):
    socket = SocketIO()
    if timeout:
        socket.settimeout(timeout)
    socket.connect(address, **extra)
    return socket


class Event(object):
    def __init__(self, name, data=None):
        self.name = name
        self.data = data


class SocketIO(WebSocket):
    def __init__(self):
        super(SocketIO, self).__init__()
        self.secure = False
        self.message_id = 1

    def encode(self, endpoint, message):
        if isinstance(message, basestring):
            message = "3:{0}:{1}:{2}".format(
                self.message_id, endpoint, message)
        elif isinstance(message, Event):
            message = "5:{0}:{1}:{2}".format(
                self.message_id, endpoint, json.dumps({
                    "name": message.name,
                    "args": [message.data]
                }))
        elif isinstance(message, (object, dict)):
            message = "4:{0}:{1}:{2}".format(
                self.message_id, endpoint, json.dumps(message))
        else:
            raise SocketIOException("Can't encode message")
        return message

    def connect(self, address, **extra):
        host, port = address
        http = httplib.HTTPConnection(host, port)
        http.request('POST', '/socket.io/1/')
        response = http.getresponse()
        if not response.status == 200:
            raise SocketIOException(response.reason)
        key = response.read().split(':')[0]
        return super(SocketIO, self).connect((
            host, port, '/socket.io/1/websocket/{0}'.format(
                key)
        ))

    def decode(self, data):
        data.encode('utf-8', 'replace')
        data = urllib.unquote_plus(data)
        message_type, message_id, message_endpoint, data = data.split(":", 3)
        if message_type == 3:
            return data
        if message_type == 4:
            return json.loads(data)
        if message_type == 5:
            event = json.loads(data)
            return Event(event["name"], event["args"])
        return None

    def send(self, endpoint, data):
        message = self.encode(endpoint, data)
        self.message_id += 1
        return super(SocketIO, self).send(message)

    def recv(self):
        data = super(SocketIO, self).recv()
        return self.decode(data)
